//-----------------------------------------------------------------------
// File : MonochromeFilter.hlsl
// Desc : Monochrome Filtering
// Copyright(c) Project Asura. All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/08/16 [Pocol] VK쐬
// <End>

//-----------------------------------------------------------------------
// Constant Buffer and Global Variables
//-----------------------------------------------------------------------
Texture2D    ColorTex : register( t0 );
SamplerState ColorSmp : register( s0 );

/////////////////////////////////////////////////////////////////////////
// VSInput structure
/////////////////////////////////////////////////////////////////////////
struct VSInput
{
	float3 Position : POSITION;
	float2 TexCoord : TEXCOORD;
};

/////////////////////////////////////////////////////////////////////////
// VSOutput structure
/////////////////////////////////////////////////////////////////////////
struct VSOutput
{
	float4 Position : SV_POSITION;
	float2 TexCoord : TEXCOORD0;
};

//----------------------------------------------------------------------
//! @brief _VF[_Gg[|Cg
//----------------------------------------------------------------------
VSOutput VSFunc( VSInput input )
{
	VSOutput output = (VSOutput)0;

	output.Position = float4( input.Position, 1.0f );
	output.TexCoord = input.TexCoord;

	return output;
}

//----------------------------------------------------------------------
//! @brief sNZVF[_Gg[|Cg
//----------------------------------------------------------------------
float4 PSFunc( VSOutput input ) : SV_TARGET
{
	float4 output = ColorTex.Sample( ColorSmp, input.TexCoord );
	float4 RGBtoY = { 0.299f, 0.587f, 0.114f, 0.0f };
	float  y = dot( RGBtoY, output );

	float3 ItoRGB = { 0.956f, -0.272f, -1.108f };
	float3 QtoRGB = { 0.620f, -0.647f, 1.705f };

	output = float4( y + ItoRGB * 0.0f + QtoRGB * 0.0f, output.a );

	return output;
}