//-----------------------------------------------------------------------
// File : PrewittFilter.hlsl
// Desc : Prewitt Filtering
// Copyright(c) Project Asura. All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/08/20 [Pocol] VK쐬
// <End>

//-----------------------------------------------------------------------
// Constant Buffer and Global Variables
//-----------------------------------------------------------------------
Texture2D    ColorTex : register( t0 );
SamplerState ColorSmp : register( s0 );

/////////////////////////////////////////////////////////////////////////
// VSInput structure
/////////////////////////////////////////////////////////////////////////
struct VSInput
{
	float3 Position : POSITION;
	float2 TexCoord : TEXCOORD;
};

/////////////////////////////////////////////////////////////////////////
// VSOutput structure
/////////////////////////////////////////////////////////////////////////
struct VSOutput
{
	float4 Position : SV_POSITION;
	float2 TexCoord : TEXCOORD0;
};

//----------------------------------------------------------------------
//! @brief _VF[_Gg[|Cg
//----------------------------------------------------------------------
VSOutput VSFunc( VSInput input )
{
	VSOutput output = (VSOutput)0;

	output.Position = float4( input.Position, 1.0f );
	output.TexCoord = input.TexCoord;

	return output;
}


//----------------------------------------------------------------------
//! @brief sNZVF[_Gg[|Cg
//----------------------------------------------------------------------
float4 PSFunc( VSOutput input ) : SV_TARGET
{
    float invW = 0.0f;
    float invH = 0.0f;
    {
        float w = 0.0f;
        float h = 0.0f;
        ColorTex.GetDimensions( w, h );
        invW = 1.0f / w;
        invH = 1.0f / h;
    }
    float4 c0 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH *  0 ) ); //i
    float4 c1 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH * -1 ) ); //i
    float4 c2 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  0, invH *  1 ) ); //i
    float4 c3 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH *  0 ) ); //i
    float4 c4 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH *  0 ) ); //iE
    float4 c5 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH * -1 ) ); //i
    float4 c6 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH * -1 ) ); //iE
    float4 c7 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW * -1, invH *  1 ) ); //i
    float4 c8 = ColorTex.Sample( ColorSmp, input.TexCoord + float2( invW *  1, invH *  1 ) ); //iE

    // PrewitttB^  
    float4 output = (float4)0;
    float4 Gh = (c7 * -1.0) + (c3 * -1.0f) + (c5 * -1.0) + (c8 * 1.0f) + (c4 * 1.0f) + (c6 * 1.0f);
    float4 Gv = (c7 * -1.0) + (c2 * -1.0f) + (c8 * -1.0) + (c5 * 1.0f) + (c1 * 1.0f) + (c6 * 1.0f);
    output.r = sqrt( Gh.r * Gh.r + Gv.r * Gv.r );
    output.g = sqrt( Gh.g * Gh.g + Gv.g * Gv.g );
    output.b = sqrt( Gh.b * Gh.b + Gv.b * Gv.b );
    output.a = sqrt( Gh.a * Gh.a + Gv.a * Gv.a );

	return output;
}